﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using Utils4226;

#endregion

namespace PageUnitsSample
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            transformScaleToolStripMenuItem_Click(scaleToolStripMenuItem, EventArgs.Empty);
            ToolStripMenuItem menuItem = new ToolStripMenuItem("Scale");
            menuItem.DropDown = scaleContextMenuStrip;
            this.menuStripScale.Items.Insert(0, menuItem);

        }

        String s = "For example, it’s not important that the ruler at the top of the document I’m typing this sentence into is currently showing an inch as 1 9/16 inches. What is important is the proportion of the dimensions of each line in proportion to the units shown as “inches” as compared to the width and height of each line as I type. The principle of WYSIWYG (What You See Is What You Get) says that I should be able to print something very similar to what I’m seeing on the screen . When my word processing program shows a line wrapping at a certain word when I get close to the 6 ½ inches area inside of my margins (my page settings are set to standard 8 ½ inch wide paper with a 1 inch margin on each side), I want that same wrap to happen at the same word when I print. To make that happen, we need to be able to write a program that can wrap text at units other than pixels, as shown in Figure 1.";

        private void MainForm_Load(object sender, EventArgs e)
        {
            ShowClientSize();
        }

        private void MainForm_Paint(object sender, PaintEventArgs e)
        {

        }

        private void MainForm_Resize(object sender, EventArgs e)
        {
            ShowClientSize();
        }

        private void PaintInInches(Graphics g)
        {
            // Set page units and scale
            g.PageUnit = GraphicsUnit.Inch;
            g.PageScale = m_fScale; // 1 unit is 1 inch
            using (Font rulerFont = new Font("MS Sans Serif", 8.25f))
            using (Font textFont = new Font("Times New Roman", 11))
            {
                PaintHelper(g, rulerFont, textFont);
            }

        }

        private void TransformPaintInInches(Graphics graphics)
        {
            Matrix mat = new Matrix();
            mat.Scale(graphics.DpiX*m_fScale, graphics.DpiY*m_fScale);
            graphics.Transform = mat;
            using (Font rulerFont =
                new Font("MS Sans Serif", Conversion.Unscale(graphics.DpiY, 8.25f)))
            using (Font textFont = 
                new Font("Times New Roman", Conversion.Unscale(graphics.DpiY, 11f)))
            {
                PaintHelper(graphics, rulerFont, textFont);
            }

        }

        private void PaintHelper(Graphics g, Font rulerFont, Font textFont)
        {
            PaintHelper(g, rulerFont, textFont, this);
        }

        private void PaintHelper(Graphics g, Font rulerFont, Font textFont, Control control)
        {
            using (Pen blackPen = new Pen(Color.Black, 0))
            {
                float rulerFontHeight = rulerFont.GetHeight(g); // Inches
                // Specifiy units in inches
                RectangleF rulerRect =
                  new RectangleF(
                  0, 0,
                  6.5f, rulerFontHeight * 1.5f);
                // Draw in inches
                g.DrawRectangle(
                  blackPen,
                  rulerRect.X, rulerRect.Y,
                  rulerRect.Width, rulerRect.Height);

                // Draw 6.5 inches in inches
                for (float i = 1; i <= 6; ++i)
                {
                    // Specify units in inches
                    RectangleF rect = new RectangleF(
                        i - rulerFontHeight, 0,
                        rulerFontHeight * 2, rulerFontHeight);
                    StringFormat format = new StringFormat();
                    format.Alignment = StringAlignment.Center;
                    g.DrawString(i.ToString(), rulerFont, Brushes.Black, rect, format);
                    //g.DrawRectangle(blackPen, rect.X, rect.Y, rect.Width, rect.Height);
                    //g.DrawLine(blackPen, i, 0, i, rulerFontHeight/2);
                }

                // Draw 6.5 inches in half inches
                for (float i = 0.5f; i <= 6.5f; i += 1.0f)
                {
                    g.DrawLine(blackPen, i, 0, i, rulerFontHeight / 2);
                }
                // Draw text wrapped to 6.5 inch margin
                //Convert client rect to inches
                RectangleF textRect = new RectangleF(
                    0, rulerRect.Height,
                    6.5f, 
                    Conversion.PixelsToInches(g, control.ClientRectangle.Height) 
                        - rulerRect.Height);
                g.FillRectangle(Brushes.White, textRect);
                g.DrawRectangle(blackPen, textRect.X, textRect.Y, 
                    textRect.Width, textRect.Height);
                g.DrawString(s, textFont, Brushes.Black, textRect);
            }
        }

        private void ManuallyPaintInInches(Graphics g)
        {
            ManuallyPaintInInches(g, this);
        }

        private void ManuallyPaintInInches(Graphics g, Control control)
        {
            using (Font rulerFont = new Font("MS Sans Serif", 8.25f))
            {
                float rulerFontHeightInches = Conversion.PixelsToInches(g, rulerFont.GetHeight(g));
                // Specifiy units in inches
                RectangleF rulerRect =
                  new RectangleF(
                  0, 0,
                  6.5f, rulerFontHeightInches * 1.5f);
                // Convert inches to pixels
                g.DrawRectangle(
                  Pens.Black,
                  Conversion.InchesToPixels(g, rulerRect.X),
                  Conversion.InchesToPixels(g, rulerRect.Y),
                  Conversion.InchesToPixels(g, rulerRect.Width), 
                  Conversion.InchesToPixels(g, rulerRect.Height));

                // Draw 6.5 inches in inches
                for (float i = 1; i <= 6; ++i)
                {
                    // Specify units in inches, make rectange to draw inch number
                    RectangleF rect = new RectangleF(
                        i - rulerFontHeightInches, 0,
                        rulerFontHeightInches * 2, rulerFontHeightInches);
                    StringFormat format = new StringFormat();
                    format.Alignment = StringAlignment.Center;
                    //Draw inch number centered in rectangle
                    g.DrawString(
                        i.ToString(), rulerFont, Brushes.Black,
                        Conversion.InchesToPixels(g, rect), format);
                }

                // Draw 6.5 inches in half inches
                for (float i = 0.5f; i <= 6.5f; i += 1.0f)
                {
                    g.DrawLine(Pens.Black,
                        Conversion.InchesToPixels(g, i), 
                        0,
                        Conversion.InchesToPixels(g, i),
                        Conversion.InchesToPixels(g, rulerFontHeightInches / 2));
                }

                //Draw the text inside a rectangle, specified in inches.
                using (Font textFont = new Font("Times New Roman", 11))
                {
                    // Draw text wrapped to 6.5 inch margin
                    RectangleF textRect = 
                        new RectangleF(0, rulerRect.Height, 6.5f, 
                            Conversion.PixelsToInches(g, control.ClientRectangle.Height) - rulerRect.Height);
                    g.FillRectangle(Brushes.White, 
                        Conversion.InchesToPixels(g, textRect));
                    g.DrawRectangle(Pens.Black,
                        Conversion.InchesToPixels(g, textRect.X),
                        Conversion.InchesToPixels(g, textRect.Y),
                        Conversion.InchesToPixels(g, textRect.Width),
                        Conversion.InchesToPixels(g, textRect.Height));
                    g.DrawString(s, textFont, Brushes.Black,
                        Conversion.InchesToPixels(g, textRect));
                }
            }

        }
        private void ShowClientSize()
        {
            ShowClientSize(this);
        }


        private void ShowClientSize(Control control)
        {
            using (Graphics g = control.CreateGraphics())
            {
                // Set page unit to inches
                g.PageUnit = GraphicsUnit.Inch;
                g.PageScale = m_fScale;
                PointF[] bottomRight =
                  new PointF[] { new PointF(control.ClientSize.Width,control.ClientSize.Height) };
                // Convert client size from pixels to inches
                g.TransformPoints(
                  CoordinateSpace.Page, CoordinateSpace.Device, bottomRight);
                this.toolStripStatusLabelInfo.Text = string.Format("ClientSize= {0}x{1} pixels ({2}x{3} inches)", this.ClientSize.Width, this.ClientSize.Height, bottomRight[0].X, bottomRight[0].Y);
            }
        }

        float m_fScale = 1.0f;
        private void zoom100ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            m_fScale = (float)Double.Parse((String)((ToolStripMenuItem)sender).Tag);
            Invalidate(true);
        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            zoomToolStripMenuItem.Enabled = scaleToolStripMenuItem.Checked;
            transformToolStripMenu.Enabled = transformScaleToolStripMenuItem.Checked;
        }

        private void transformScaleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ClearChecksAndEnable();
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            toolStripStatusLabelMode.Text = (String)item.Tag;
            item.Enabled = false;
            item.Checked = true;
            Invalidate(true);
        }

        private void ClearChecksAndEnable() {
            transformScaleToolStripMenuItem.Checked = false;
            manualToolStripMenuItem.Checked = false;
            scaleToolStripMenuItem.Checked = false;
            transformScaleToolStripMenuItem.Enabled = true;
            manualToolStripMenuItem.Enabled = true;
            scaleToolStripMenuItem.Enabled = true;
        }

        private void toolStripMenuItemTran50_Click(object sender, EventArgs e)
        {
            m_fScale = (float)Double.Parse((String)((ToolStripMenuItem)sender).Tag);
            Invalidate(true);
        }

        private void panelScale_Paint(object sender, PaintEventArgs e)
        {
            if (manualToolStripMenuItem.Checked)
            {
                ManuallyPaintInInches(e.Graphics);
            }
            else if (transformScaleToolStripMenuItem.Checked)
            {
                TransformPaintInInches(e.Graphics);
            }
            else
            {
                PaintInInches(e.Graphics);
            }
        }

        private void tabPageTransform_Click(object sender, EventArgs e)
        {

        }

        RectangleF rectHit = new RectangleF(0.5f, 0.5f, 3, 2);
        Brush brushRect = Brushes.Aquamarine;
 
        private void tabPageTransform_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            GraphicsState state = g.Save();
            SetScale(g);
            g.FillRectangle(brushRect, rectHit.Left, rectHit.Top, rectHit.Width, rectHit.Height);
            g.Restore(state);
        }

        private void tabPageTransform_MouseEnter(object sender, EventArgs e)
        {
            
        }

        private void tabPageTransform_MouseLeave(object sender, EventArgs e)
        {
            
        }

        private void tabPageTransform_MouseDown(object sender, MouseEventArgs e)
        {
            PointF[] pts = new PointF[] { e.Location };
            if (checkBoxTransform.Checked)
            {
                using (Graphics g = tabPageTransform.CreateGraphics())
                {
                    SetScale(g);
                    g.TransformPoints(CoordinateSpace.Page, CoordinateSpace.Device, pts);
                }
            }
            toolStripStatusLabelInfo.Text = String.Format(
                "Mouse Down {0}; Rectangle Top Left {1}", pts[0], rectHit.Location);
            if (rectHit.Contains(pts[0]))
            {
                brushRect = Brushes.AliceBlue;
                Invalidate(true);
            }
        }

        private void SetScale(Graphics g)
        {
            g.PageUnit = GraphicsUnit.Inch;
            g.PageScale = 1f;
        }

        private void tabPageTransform_MouseUp(object sender, MouseEventArgs e)
        {
            brushRect = Brushes.Aquamarine;
            Invalidate(true);
        }

        private void checkBoxTransform_CheckedChanged(object sender, EventArgs e)
        {

        }
    }
}